@@ -71,7 +71,7 @@
 #include <stdio.h>
 #include <stdint.h>
 #include <inttypes.h>
-#include <libzfs_impl.h>
+#include <libzfs.h>
 
 #define	POOL_MEASUREMENT	"zpool_stats"
 #define	SCAN_MEASUREMENT	"zpool_scan_stats"
@@ -90,7 +90,7 @@ char metric_data_type = 'u';
 uint64_t metric_value_mask = UINT64_MAX;
 uint64_t timestamp = 0;
 int complained_about_sync = 0;
-char *tags = "";
+const char *tags = "";
 
 typedef int (*stat_printer_f)(nvlist_t *, const char *, const char *);
 
@@ -101,9 +101,10 @@ typedef int (*stat_printer_f)(nvlist_t *, const char *, const char *);
  * caller is responsible for freeing result
  */
 static char *
-escape_string(char *s)
+escape_string(const char *s)
 {
-	char *c, *d;
+	const char *c;
+	char *d;
 	char *t = (char *)malloc(ZFS_MAX_DATASET_NAME_LEN * 2);
 	if (t == NULL) {
 		fprintf(stderr, "error: cannot allocate memory\n");
@@ -117,7 +118,7 @@ escape_string(char *s)
 		case '=':
 		case '\\':
 			*d++ = '\\';
-			fallthrough;
+			zfs_fallthrough;
 		default:
 			*d = *c;
 		}
@@ -130,7 +131,7 @@ escape_string(char *s)
  * print key=value where value is a uint64_t
  */
 static void
-print_kv(char *key, uint64_t value)
+print_kv(const char *key, uint64_t value)
 {
 	printf("%s=%llu%c", key,
 	    (u_longlong_t)value & metric_value_mask, metric_data_type);
@@ -151,9 +152,9 @@ print_scan_status(nvlist_t *nvroot, const char *pool_name)
 	uint64_t remaining_time;
 	pool_scan_stat_t *ps = NULL;
 	double pct_done;
-	char *state[DSS_NUM_STATES] = {
+	const char *const state[DSS_NUM_STATES] = {
 	    "none", "scanning", "finished", "canceled"};
-	char *func;
+	const char *func;
 
 	(void) nvlist_lookup_uint64_array(nvroot,
 	    ZPOOL_CONFIG_SCAN_STATS,
@@ -237,6 +238,7 @@ print_scan_status(nvlist_t *nvroot, const char *pool_name)
 	print_kv("end_ts", ps->pss_end_time);
 	print_kv(",errors", ps->pss_errors);
 	print_kv(",examined", examined);
+	print_kv(",skipped", ps->pss_skipped);
 	print_kv(",issued", ps->pss_issued);
 	print_kv(",pass_examined", pass_exam);
 	print_kv(",pass_issued", ps->pss_pass_issued);
@@ -248,7 +250,6 @@ print_scan_status(nvlist_t *nvroot, const char *pool_name)
 	print_kv(",remaining_t", remaining_time);
 	print_kv(",start_ts", ps->pss_start_time);
 	print_kv(",to_examine", ps->pss_to_examine);
-	print_kv(",to_process", ps->pss_to_process);
 	printf(" %llu\n", (u_longlong_t)timestamp);
 	return (0);
 }
@@ -261,23 +262,21 @@ static char *
 get_vdev_name(nvlist_t *nvroot, const char *parent_name)
 {
 	static char vdev_name[256];
-	char *vdev_type = NULL;
 	uint64_t vdev_id = 0;
 
-	if (nvlist_lookup_string(nvroot, ZPOOL_CONFIG_TYPE,
-	    &vdev_type) != 0) {
-		vdev_type = "unknown";
-	}
+	const char *vdev_type = "unknown";
+	(void) nvlist_lookup_string(nvroot, ZPOOL_CONFIG_TYPE, &vdev_type);
+
 	if (nvlist_lookup_uint64(
-	    nvroot, ZPOOL_CONFIG_ID, &vdev_id) != 0) {
+	    nvroot, ZPOOL_CONFIG_ID, &vdev_id) != 0)
 		vdev_id = UINT64_MAX;
-	}
+
 	if (parent_name == NULL) {
 		(void) snprintf(vdev_name, sizeof (vdev_name), "%s",
 		    vdev_type);
 	} else {
 		(void) snprintf(vdev_name, sizeof (vdev_name),
-		    "%s/%s-%llu",
+		    "%.220s/%s-%llu",
 		    parent_name, vdev_type, (u_longlong_t)vdev_id);
 	}
 	return (vdev_name);
@@ -297,22 +296,15 @@ static char *
 get_vdev_desc(nvlist_t *nvroot, const char *parent_name)
 {
 	static char vdev_desc[2 * MAXPATHLEN];
-	char *vdev_type = NULL;
-	uint64_t vdev_id = 0;
 	char vdev_value[MAXPATHLEN];
-	char *vdev_path = NULL;
 	char *s, *t;
 
-	if (nvlist_lookup_string(nvroot, ZPOOL_CONFIG_TYPE, &vdev_type) != 0) {
-		vdev_type = "unknown";
-	}
-	if (nvlist_lookup_uint64(nvroot, ZPOOL_CONFIG_ID, &vdev_id) != 0) {
-		vdev_id = UINT64_MAX;
-	}
-	if (nvlist_lookup_string(
-	    nvroot, ZPOOL_CONFIG_PATH, &vdev_path) != 0) {
-		vdev_path = NULL;
-	}
+	const char *vdev_type = "unknown";
+	uint64_t vdev_id = UINT64_MAX;
+	const char *vdev_path = NULL;
+	(void) nvlist_lookup_string(nvroot, ZPOOL_CONFIG_TYPE, &vdev_type);
+	(void) nvlist_lookup_uint64(nvroot, ZPOOL_CONFIG_ID, &vdev_id);
+	(void) nvlist_lookup_string(nvroot, ZPOOL_CONFIG_PATH, &vdev_path);
 
 	if (parent_name == NULL) {
 		s = escape_string(vdev_type);
@@ -392,8 +384,8 @@ print_vdev_latency_stats(nvlist_t *nvroot, const char *pool_name,
 
 	/* short_names become part of the metric name and are influxdb-ready */
 	struct lat_lookup {
-	    char *name;
-	    char *short_name;
+	    const char *name;
+	    const char *short_name;
 	    uint64_t sum;
 	    uint64_t *array;
 	};
@@ -410,6 +402,7 @@ print_vdev_latency_stats(nvlist_t *nvroot, const char *pool_name,
 #ifdef ZPOOL_CONFIG_VDEV_TRIM_LAT_HISTO
 	    {ZPOOL_CONFIG_VDEV_TRIM_LAT_HISTO,    "trim", 0},
 #endif
+	    {ZPOOL_CONFIG_VDEV_REBUILD_LAT_HISTO,    "rebuild", 0},
 	    {NULL,	NULL}
 	};
 
@@ -485,8 +478,8 @@ print_vdev_size_stats(nvlist_t *nvroot, const char *pool_name,
 
 	/* short_names become the field name */
 	struct size_lookup {
-	    char *name;
-	    char *short_name;
+	    const char *name;
+	    const char *short_name;
 	    uint64_t sum;
 	    uint64_t *array;
 	};
@@ -505,6 +498,8 @@ print_vdev_size_stats(nvlist_t *nvroot, const char *pool_name,
 	    {ZPOOL_CONFIG_VDEV_IND_TRIM_HISTO,    "trim_write_ind"},
 	    {ZPOOL_CONFIG_VDEV_AGG_TRIM_HISTO,    "trim_write_agg"},
 #endif
+	    {ZPOOL_CONFIG_VDEV_IND_REBUILD_HISTO,    "rebuild_write_ind"},
+	    {ZPOOL_CONFIG_VDEV_AGG_REBUILD_HISTO,    "rebuild_write_agg"},
 	    {NULL,	NULL}
 	};
 
@@ -575,8 +570,8 @@ print_queue_stats(nvlist_t *nvroot, const char *pool_name,
 
 	/* short_names are used for the field name */
 	struct queue_lookup {
-	    char *name;
-	    char *short_name;
+	    const char *name;
+	    const char *short_name;
 	};
 	struct queue_lookup queue_type[] = {
 	    {ZPOOL_CONFIG_VDEV_SYNC_R_ACTIVE_QUEUE,	"sync_r_active"},
@@ -584,11 +579,13 @@ print_queue_stats(nvlist_t *nvroot, const char *pool_name,
 	    {ZPOOL_CONFIG_VDEV_ASYNC_R_ACTIVE_QUEUE,	"async_r_active"},
 	    {ZPOOL_CONFIG_VDEV_ASYNC_W_ACTIVE_QUEUE,	"async_w_active"},
 	    {ZPOOL_CONFIG_VDEV_SCRUB_ACTIVE_QUEUE,	"async_scrub_active"},
+	    {ZPOOL_CONFIG_VDEV_REBUILD_ACTIVE_QUEUE,	"rebuild_active"},
 	    {ZPOOL_CONFIG_VDEV_SYNC_R_PEND_QUEUE,	"sync_r_pend"},
 	    {ZPOOL_CONFIG_VDEV_SYNC_W_PEND_QUEUE,	"sync_w_pend"},
 	    {ZPOOL_CONFIG_VDEV_ASYNC_R_PEND_QUEUE,	"async_r_pend"},
 	    {ZPOOL_CONFIG_VDEV_ASYNC_W_PEND_QUEUE,	"async_w_pend"},
 	    {ZPOOL_CONFIG_VDEV_SCRUB_PEND_QUEUE,	"async_scrub_pend"},
+	    {ZPOOL_CONFIG_VDEV_REBUILD_PEND_QUEUE,	"rebuild_pend"},
 	    {NULL,	NULL}
 	};
 
@@ -626,8 +623,8 @@ print_top_level_vdev_stats(nvlist_t *nvroot, const char *pool_name)
 
 	/* short_names become part of the metric name */
 	struct queue_lookup {
-	    char *name;
-	    char *short_name;
+	    const char *name;
+	    const char *short_name;
 	};
 	struct queue_lookup queue_type[] = {
 	    {ZPOOL_CONFIG_VDEV_SYNC_R_ACTIVE_QUEUE, "sync_r_active_queue"},
@@ -635,11 +632,13 @@ print_top_level_vdev_stats(nvlist_t *nvroot, const char *pool_name)
 	    {ZPOOL_CONFIG_VDEV_ASYNC_R_ACTIVE_QUEUE, "async_r_active_queue"},
 	    {ZPOOL_CONFIG_VDEV_ASYNC_W_ACTIVE_QUEUE, "async_w_active_queue"},
 	    {ZPOOL_CONFIG_VDEV_SCRUB_ACTIVE_QUEUE, "async_scrub_active_queue"},
+	    {ZPOOL_CONFIG_VDEV_REBUILD_ACTIVE_QUEUE, "rebuild_active_queue"},
 	    {ZPOOL_CONFIG_VDEV_SYNC_R_PEND_QUEUE, "sync_r_pend_queue"},
 	    {ZPOOL_CONFIG_VDEV_SYNC_W_PEND_QUEUE, "sync_w_pend_queue"},
 	    {ZPOOL_CONFIG_VDEV_ASYNC_R_PEND_QUEUE, "async_r_pend_queue"},
 	    {ZPOOL_CONFIG_VDEV_ASYNC_W_PEND_QUEUE, "async_w_pend_queue"},
 	    {ZPOOL_CONFIG_VDEV_SCRUB_PEND_QUEUE, "async_scrub_pend_queue"},
+	    {ZPOOL_CONFIG_VDEV_REBUILD_PEND_QUEUE, "rebuild_pend_queue"},
 	    {NULL, NULL}
 	};
 
@@ -688,8 +687,10 @@ print_recursive_stats(stat_printer_f func, nvlist_t *nvroot,
 		    sizeof (vdev_name));
 
 		for (c = 0; c < children; c++) {
-			print_recursive_stats(func, child[c], pool_name,
+			err = print_recursive_stats(func, child[c], pool_name,
 			    vdev_name, descend);
+			if (err)
+				return (err);
 		}
 	}
 	return (0);
@@ -714,7 +715,7 @@ print_stats(zpool_handle_t *zhp, void *data)
 
 	/* if not this pool return quickly */
 	if (data &&
-	    strncmp(data, zhp->zpool_name, ZFS_MAX_DATASET_NAME_LEN) != 0) {
+	    strncmp(data, zpool_get_name(zhp), ZFS_MAX_DATASET_NAME_LEN) != 0) {
 		zpool_close(zhp);
 		return (0);
 	}
@@ -742,7 +743,7 @@ print_stats(zpool_handle_t *zhp, void *data)
 		return (3);
 	}
 
-	pool_name = escape_string(zhp->zpool_name);
+	pool_name = escape_string(zpool_get_name(zhp));
 	err = print_recursive_stats(print_summary_stats, nvroot,
 	    pool_name, NULL, 1);
 	/* if any of these return an error, skip the rest */
@@ -781,7 +782,7 @@ main(int argc, char *argv[])
 {
 	int opt;
 	int ret = 8;
-	char *line = NULL;
+	char *line = NULL, *ttags = NULL;
 	size_t len, tagslen = 0;
 	struct option long_options[] = {
 	    {"execd", no_argument, NULL, 'e'},
@@ -809,15 +810,17 @@ main(int argc, char *argv[])
 			sum_histogram_buckets = 1;
 			break;
 		case 't':
+			free(ttags);
 			tagslen = strlen(optarg) + 2;
-			tags = calloc(tagslen, 1);
-			if (tags == NULL) {
+			ttags = calloc(1, tagslen);
+			if (ttags == NULL) {
 				fprintf(stderr,
 				    "error: cannot allocate memory "
 				    "for tags\n");
 				exit(1);
 			}
-			(void) snprintf(tags, tagslen, ",%s", optarg);
+			(void) snprintf(ttags, tagslen, ",%s", optarg);
+			tags = ttags;
 			break;
 		default:
 			usage(argv[0]);
