@@ -6,7 +6,7 @@
  * You may not use this file except in compliance with the License.
  *
  * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
- * or http://www.opensolaris.org/os/licensing.
+ * or https://opensource.org/licenses/CDDL-1.0.
  * See the License for the specific language governing permissions
  * and limitations under the License.
  *
@@ -22,7 +22,7 @@
 #ifndef	_SYS_FS_ZFS_DELAY_H
 #define	_SYS_FS_ZFS_DELAY_H
 
-#include <linux/delay_compat.h>
+#include <sys/timer.h>
 
 /*
  * Generic wrapper to sleep until a given time.
