@@ -6,7 +6,7 @@
  * You may not use this file except in compliance with the License.
  *
  * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
- * or http://www.opensolaris.org/os/licensing.
+ * or https://opensource.org/licenses/CDDL-1.0.
  * See the License for the specific language governing permissions
  * and limitations under the License.
  *
@@ -26,12 +26,13 @@
 #ifndef _SYS_ZFS_RACCT_H
 #define	_SYS_ZFS_RACCT_H
 
-#include <sys/zfs_context.h>
+#include <sys/types.h>
+#include <sys/spa.h>
 
 /*
  * Platform-dependent resource accounting hooks
  */
-void zfs_racct_read(uint64_t size, uint64_t iops);
-void zfs_racct_write(uint64_t size, uint64_t iops);
+void zfs_racct_read(spa_t *spa, uint64_t size, uint64_t iops, uint32_t flags);
+void zfs_racct_write(spa_t *spa, uint64_t size, uint64_t iops, uint32_t flags);
 
 #endif /* _SYS_ZFS_RACCT_H */
