@@ -6,7 +6,7 @@
  * You may not use this file except in compliance with the License.
  *
  * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
- * or http://www.opensolaris.org/os/licensing.
+ * or https://opensource.org/licenses/CDDL-1.0.
  * See the License for the specific language governing permissions
  * and limitations under the License.
  *
@@ -92,39 +92,6 @@ typedef struct kstat {
 	void		*ks_lock;	/* protects this kstat's data */
 } kstat_t;
 
-#ifdef _SYSCALL32
-
-typedef int32_t kid32_t;
-
-typedef struct kstat32 {
-	/*
-	 * Fields relevant to both kernel and user
-	 */
-	hrtime_t	ks_crtime;
-	caddr32_t	ks_next;		/* struct kstat pointer */
-	kid32_t		ks_kid;
-	char		ks_module[KSTAT_STRLEN];
-	uint8_t		ks_resv;
-	int32_t		ks_instance;
-	char		ks_name[KSTAT_STRLEN];
-	uint8_t		ks_type;
-	char		ks_class[KSTAT_STRLEN];
-	uint8_t		ks_flags;
-	caddr32_t	ks_data;		/* type-specific data */
-	uint32_t	ks_ndata;
-	size32_t	ks_data_size;
-	hrtime_t	ks_snaptime;
-	/*
-	 * Fields relevant to kernel only (only needed here for padding)
-	 */
-	int32_t		_ks_update;
-	caddr32_t	_ks_private;
-	int32_t		_ks_snapshot;
-	caddr32_t	_ks_lock;
-} kstat32_t;
-
-#endif	/* _SYSCALL32 */
-
 /*
  * kstat structure and locking strategy
  *
@@ -383,9 +350,9 @@ typedef struct kstat32 {
  *
  *	ksp->ks_snaptime = gethrtime();
  *	if (rw == KSTAT_WRITE)
- *		bcopy(buf, ksp->ks_data, ksp->ks_data_size);
+ *		memcpy(ksp->ks_data, buf, ksp->ks_data_size);
  *	else
- *		bcopy(ksp->ks_data, buf, ksp->ks_data_size);
+ *		memcpy(buf, ksp->ks_data, ksp->ks_data_size);
  *	return (0);
  *
  * A more illuminating example is taking a snapshot of a linked list:
@@ -394,7 +361,7 @@ typedef struct kstat32 {
  *	if (rw == KSTAT_WRITE)
  *		return (EACCES);		... See below ...
  *	for (foo = first_foo; foo; foo = foo->next) {
- *		bcopy((char *) foo, (char *) buf, sizeof (struct foo));
+ *		memcpy(buf, foo, sizeof (struct foo));
  *		buf = ((struct foo *) buf) + 1;
  *	}
  *	return (0);
@@ -423,12 +390,12 @@ typedef struct kstat32 {
  * 	uint_t i;
  *
  * 	... Do the regular copy ...
- * 	bcopy(ksp->ks_data, buf, sizeof (kstat_named_t) * ksp->ks_ndata);
+ * 	memcpy(buf, ksp->ks_data, sizeof (kstat_named_t) * ksp->ks_ndata);
  *
  * 	for (i = 0; i < ksp->ks_ndata; i++, knp++) {
  *		if (knp[i].data_type == KSTAT_DATA_STRING &&
  *		    KSTAT_NAMED_STR_PTR(knp) != NULL) {
- *			bcopy(KSTAT_NAMED_STR_PTR(knp), end,
+ *			memcpy(end, KSTAT_NAMED_STR_PTR(knp),
  *			    KSTAT_NAMED_STR_BUFLEN(knp));
  *			KSTAT_NAMED_STR_PTR(knp) = end;
  *			end += KSTAT_NAMED_STR_BUFLEN(knp);
@@ -467,7 +434,7 @@ typedef struct kstat_named {
  * 64-bit compilation environments or 32-bit non-maximally conformant
  * C89 or C90 ANSI C compilation environments (cc -Xt and cc -Xa). In the
  * C99 ANSI C compilation environment, the long long type is supported.
- * The _INT64_TYPE is defined by the implementation (see sys/int_types.h).
+ * The _INT64_TYPE is defined by the implementation (see sys/inttypes.h).
  */
 #if defined(_INT64_TYPE)
 		int64_t		i64;
