@@ -592,4 +592,7 @@ zfs_isa207_available(void)
 
 #endif
 
+extern void simd_stat_init(void);
+extern void simd_stat_fini(void);
+
 #endif /* _LIBSPL_SYS_SIMD_H */
