@@ -6,7 +6,7 @@
  * You may not use this file except in compliance with the License.
  *
  * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
- * or http://www.opensolaris.org/os/licensing.
+ * or https://opensource.org/licenses/CDDL-1.0.
  * See the License for the specific language governing permissions
  * and limitations under the License.
  *
@@ -65,11 +65,6 @@ typedef	int32_t		ssize32_t;
 typedef	int32_t		time32_t;
 typedef	int32_t		clock32_t;
 
-struct timeval32 {
-	time32_t	tv_sec;		/* seconds */
-	int32_t		tv_usec;	/* and microseconds */
-};
-
 typedef struct timespec32 {
 	time32_t	tv_sec;		/* seconds */
 	int32_t		tv_nsec;	/* and nanoseconds */
