@@ -7,7 +7,7 @@
  * with the License.
  *
  * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
- * or http://www.opensolaris.org/os/licensing.
+ * or https://opensource.org/licenses/CDDL-1.0.
  * See the License for the specific language governing permissions
  * and limitations under the License.
  *
@@ -21,8 +21,9 @@
  */
 
 #include <unistd.h>
+#include <sys/param.h>
 
-size_t pagesize = 0;
+static size_t pagesize = 0;
 
 size_t
 spl_pagesize(void)
