@@ -25,9 +25,6 @@
  *
  */
 
-#include <sys/cdefs.h>
-__FBSDID("$FreeBSD$");
-
 #include <sys/param.h>
 #include <sys/buf.h>
 #include <sys/cmn_err.h>
@@ -65,6 +62,7 @@ __FBSDID("$FreeBSD$");
 #include <sys/spa_impl.h>
 #include <sys/stat.h>
 #include <sys/sunddi.h>
+#include <sys/sysctl.h>
 #include <sys/systm.h>
 #include <sys/taskqueue.h>
 #include <sys/uio.h>
@@ -113,6 +111,7 @@ static int zfs__fini(void);
 static void zfs_shutdown(void *, int);
 
 static eventhandler_tag zfs_shutdown_event_tag;
+static eventhandler_tag zfs_mountroot_event_tag;
 
 #define	ZFS_MIN_KSTACK_PAGES 4
 
@@ -307,16 +306,25 @@ zfs_modevent(module_t mod, int type, void *unused __unused)
 	switch (type) {
 	case MOD_LOAD:
 		err = zfs__init();
-		if (err == 0)
+		if (err == 0) {
 			zfs_shutdown_event_tag = EVENTHANDLER_REGISTER(
 			    shutdown_post_sync, zfs_shutdown, NULL,
 			    SHUTDOWN_PRI_FIRST);
+			zfs_mountroot_event_tag = EVENTHANDLER_REGISTER(
+			    mountroot, spa_boot_init, NULL,
+			    SI_ORDER_ANY);
+		}
 		return (err);
 	case MOD_UNLOAD:
 		err = zfs__fini();
-		if (err == 0 && zfs_shutdown_event_tag != NULL)
-			EVENTHANDLER_DEREGISTER(shutdown_post_sync,
-			    zfs_shutdown_event_tag);
+		if (err == 0) {
+			if (zfs_shutdown_event_tag != NULL)
+				EVENTHANDLER_DEREGISTER(shutdown_post_sync,
+				    zfs_shutdown_event_tag);
+			if (zfs_mountroot_event_tag != NULL)
+				EVENTHANDLER_DEREGISTER(mountroot,
+				    zfs_mountroot_event_tag);
+		}
 		return (err);
 	case MOD_SHUTDOWN:
 		return (0);
@@ -332,17 +340,12 @@ static moduledata_t zfs_mod = {
 	0
 };
 
-#ifdef _KERNEL
-EVENTHANDLER_DEFINE(mountroot, spa_boot_init, NULL, 0);
-#endif
+
+FEATURE(zfs, "OpenZFS support");
 
 DECLARE_MODULE(zfsctrl, zfs_mod, SI_SUB_CLOCKS, SI_ORDER_ANY);
 MODULE_VERSION(zfsctrl, 1);
-#if __FreeBSD_version > 1300092
 MODULE_DEPEND(zfsctrl, xdr, 1, 1, 1);
-#else
-MODULE_DEPEND(zfsctrl, krpc, 1, 1, 1);
-#endif
 MODULE_DEPEND(zfsctrl, acl_nfs4, 1, 1, 1);
 MODULE_DEPEND(zfsctrl, crypto, 1, 1, 1);
 MODULE_DEPEND(zfsctrl, zlib, 1, 1, 1);
