@@ -26,7 +26,6 @@
 #include <sys/kmem.h>
 #include <sys/vmem.h>
 
-/* BEGIN CSTYLED */
 /*
  * As a general rule kmem_alloc() allocations should be small, preferably
  * just a few pages since they must by physically contiguous.  Therefore, a
@@ -62,7 +61,6 @@ module_param(spl_kmem_alloc_max, uint, 0644);
 MODULE_PARM_DESC(spl_kmem_alloc_max,
 	"Maximum size in bytes for a kmem_alloc()");
 EXPORT_SYMBOL(spl_kmem_alloc_max);
-/* END CSTYLED */
 
 int
 kmem_debugging(void)
@@ -134,7 +132,6 @@ EXPORT_SYMBOL(kmem_strfree);
 void *
 spl_kvmalloc(size_t size, gfp_t lflags)
 {
-#ifdef HAVE_KVMALLOC
 	/*
 	 * GFP_KERNEL allocations can safely use kvmalloc which may
 	 * improve performance by avoiding a) high latency caused by
@@ -146,7 +143,6 @@ spl_kvmalloc(size_t size, gfp_t lflags)
 	 */
 	if ((lflags & GFP_KERNEL) == GFP_KERNEL)
 		return (kvmalloc(size, lflags));
-#endif
 
 	gfp_t kmalloc_lflags = lflags;
 
