@@ -153,19 +153,20 @@ static int
 zpl_xattr_readdir(struct inode *dxip, xattr_filldir_t *xf)
 {
 	zap_cursor_t zc;
-	zap_attribute_t	zap;
+	zap_attribute_t	*zap = zap_attribute_alloc();
 	int error;
 
 	zap_cursor_init(&zc, ITOZSB(dxip)->z_os, ITOZ(dxip)->z_id);
 
-	while ((error = -zap_cursor_retrieve(&zc, &zap)) == 0) {
+	while ((error = -zap_cursor_retrieve(&zc, zap)) == 0) {
 
-		if (zap.za_integer_length != 8 || zap.za_num_integers != 1) {
+		if (zap->za_integer_length != 8 || zap->za_num_integers != 1) {
 			error = -ENXIO;
 			break;
 		}
 
-		error = zpl_xattr_filldir(xf, zap.za_name, strlen(zap.za_name));
+		error = zpl_xattr_filldir(xf, zap->za_name,
+		    strlen(zap->za_name));
 		if (error)
 			break;
 
@@ -173,6 +174,7 @@ zpl_xattr_readdir(struct inode *dxip, xattr_filldir_t *xf)
 	}
 
 	zap_cursor_fini(&zc);
+	zap_attribute_free(zap);
 
 	if (error == -ENOENT)
 		error = 0;
@@ -712,10 +714,6 @@ __zpl_xattr_user_get(struct inode *ip, const char *name,
 {
 	int error;
 	/* xattr_resolve_name will do this for us if this is defined */
-#ifndef HAVE_XATTR_HANDLER_NAME
-	if (strcmp(name, "") == 0)
-		return (-EINVAL);
-#endif
 	if (ZFS_XA_NS_PREFIX_FORBIDDEN(name))
 		return (-EINVAL);
 	if (!(ITOZSB(ip)->z_flags & ZSB_XATTR))
@@ -745,10 +743,6 @@ __zpl_xattr_user_set(zidmap_t *user_ns,
 	(void) user_ns;
 	int error = 0;
 	/* xattr_resolve_name will do this for us if this is defined */
-#ifndef HAVE_XATTR_HANDLER_NAME
-	if (strcmp(name, "") == 0)
-		return (-EINVAL);
-#endif
 	if (ZFS_XA_NS_PREFIX_FORBIDDEN(name))
 		return (-EINVAL);
 	if (!(ITOZSB(ip)->z_flags & ZSB_XATTR))
@@ -835,10 +829,6 @@ __zpl_xattr_trusted_get(struct inode *ip, const char *name,
 	if (!capable(CAP_SYS_ADMIN))
 		return (-EACCES);
 	/* xattr_resolve_name will do this for us if this is defined */
-#ifndef HAVE_XATTR_HANDLER_NAME
-	if (strcmp(name, "") == 0)
-		return (-EINVAL);
-#endif
 	xattr_name = kmem_asprintf("%s%s", XATTR_TRUSTED_PREFIX, name);
 	error = zpl_xattr_get(ip, xattr_name, value, size);
 	kmem_strfree(xattr_name);
@@ -859,10 +849,6 @@ __zpl_xattr_trusted_set(zidmap_t *user_ns,
 	if (!capable(CAP_SYS_ADMIN))
 		return (-EACCES);
 	/* xattr_resolve_name will do this for us if this is defined */
-#ifndef HAVE_XATTR_HANDLER_NAME
-	if (strcmp(name, "") == 0)
-		return (-EINVAL);
-#endif
 	xattr_name = kmem_asprintf("%s%s", XATTR_TRUSTED_PREFIX, name);
 	error = zpl_xattr_set(ip, xattr_name, value, size, flags);
 	kmem_strfree(xattr_name);
@@ -905,10 +891,6 @@ __zpl_xattr_security_get(struct inode *ip, const char *name,
 	char *xattr_name;
 	int error;
 	/* xattr_resolve_name will do this for us if this is defined */
-#ifndef HAVE_XATTR_HANDLER_NAME
-	if (strcmp(name, "") == 0)
-		return (-EINVAL);
-#endif
 	xattr_name = kmem_asprintf("%s%s", XATTR_SECURITY_PREFIX, name);
 	error = zpl_xattr_get(ip, xattr_name, value, size);
 	kmem_strfree(xattr_name);
@@ -926,10 +908,6 @@ __zpl_xattr_security_set(zidmap_t *user_ns,
 	char *xattr_name;
 	int error;
 	/* xattr_resolve_name will do this for us if this is defined */
-#ifndef HAVE_XATTR_HANDLER_NAME
-	if (strcmp(name, "") == 0)
-		return (-EINVAL);
-#endif
 	xattr_name = kmem_asprintf("%s%s", XATTR_SECURITY_PREFIX, name);
 	error = zpl_xattr_set(ip, xattr_name, value, size, flags);
 	kmem_strfree(xattr_name);
@@ -1049,15 +1027,14 @@ zpl_set_acl_impl(struct inode *ip, struct posix_acl *acl, int type)
 
 	if (!error) {
 		if (acl)
-			zpl_set_cached_acl(ip, type, acl);
+			set_cached_acl(ip, type, acl);
 		else
-			zpl_forget_cached_acl(ip, type);
+			forget_cached_acl(ip, type);
 	}
 
 	return (error);
 }
 
-#ifdef HAVE_SET_ACL
 int
 #ifdef HAVE_SET_ACL_USERNS
 zpl_set_acl(struct user_namespace *userns, struct inode *ip,
@@ -1080,7 +1057,6 @@ zpl_set_acl(struct inode *ip, struct posix_acl *acl, int type)
 	return (zpl_set_acl_impl(ip, acl, type));
 #endif /* HAVE_SET_ACL_USERNS_DENTRY_ARG2 */
 }
-#endif /* HAVE_SET_ACL */
 
 static struct posix_acl *
 zpl_get_acl_impl(struct inode *ip, int type)
@@ -1089,17 +1065,6 @@ zpl_get_acl_impl(struct inode *ip, int type)
 	void *value = NULL;
 	char *name;
 
-	/*
-	 * As of Linux 3.14, the kernel get_acl will check this for us.
-	 * Also as of Linux 4.7, comparing against ACL_NOT_CACHED is wrong
-	 * as the kernel get_acl will set it to temporary sentinel value.
-	 */
-#ifndef HAVE_KERNEL_GET_ACL_HANDLE_CACHE
-	acl = get_cached_acl(ip, type);
-	if (acl != ACL_NOT_CACHED)
-		return (acl);
-#endif
-
 	switch (type) {
 	case ACL_TYPE_ACCESS:
 		name = XATTR_NAME_POSIX_ACL_ACCESS;
@@ -1128,12 +1093,6 @@ zpl_get_acl_impl(struct inode *ip, int type)
 	if (size > 0)
 		kmem_free(value, size);
 
-	/* As of Linux 4.7, the kernel get_acl will set this for us */
-#ifndef HAVE_KERNEL_GET_ACL_HANDLE_CACHE
-	if (!IS_ERR(acl))
-		zpl_set_cached_acl(ip, type, acl);
-#endif
-
 	return (acl);
 }
 
@@ -1270,10 +1229,6 @@ __zpl_xattr_acl_get_access(struct inode *ip, const char *name,
 	int type = ACL_TYPE_ACCESS;
 	int error;
 	/* xattr_resolve_name will do this for us if this is defined */
-#ifndef HAVE_XATTR_HANDLER_NAME
-	if (strcmp(name, "") != 0)
-		return (-EINVAL);
-#endif
 	if (ITOZSB(ip)->z_acl_type != ZFS_ACLTYPE_POSIX)
 		return (-EOPNOTSUPP);
 
@@ -1298,10 +1253,6 @@ __zpl_xattr_acl_get_default(struct inode *ip, const char *name,
 	int type = ACL_TYPE_DEFAULT;
 	int error;
 	/* xattr_resolve_name will do this for us if this is defined */
-#ifndef HAVE_XATTR_HANDLER_NAME
-	if (strcmp(name, "") != 0)
-		return (-EINVAL);
-#endif
 	if (ITOZSB(ip)->z_acl_type != ZFS_ACLTYPE_POSIX)
 		return (-EOPNOTSUPP);
 
@@ -1327,10 +1278,6 @@ __zpl_xattr_acl_set_access(zidmap_t *mnt_ns,
 	int type = ACL_TYPE_ACCESS;
 	int error = 0;
 	/* xattr_resolve_name will do this for us if this is defined */
-#ifndef HAVE_XATTR_HANDLER_NAME
-	if (strcmp(name, "") != 0)
-		return (-EINVAL);
-#endif
 	if (ITOZSB(ip)->z_acl_type != ZFS_ACLTYPE_POSIX)
 		return (-EOPNOTSUPP);
 
@@ -1348,7 +1295,7 @@ __zpl_xattr_acl_set_access(zidmap_t *mnt_ns,
 		if (IS_ERR(acl))
 			return (PTR_ERR(acl));
 		else if (acl) {
-			error = zpl_posix_acl_valid(ip, acl);
+			error = posix_acl_valid(ip->i_sb->s_user_ns, acl);
 			if (error) {
 				zpl_posix_acl_release(acl);
 				return (error);
@@ -1373,10 +1320,6 @@ __zpl_xattr_acl_set_default(zidmap_t *mnt_ns,
 	int type = ACL_TYPE_DEFAULT;
 	int error = 0;
 	/* xattr_resolve_name will do this for us if this is defined */
-#ifndef HAVE_XATTR_HANDLER_NAME
-	if (strcmp(name, "") != 0)
-		return (-EINVAL);
-#endif
 	if (ITOZSB(ip)->z_acl_type != ZFS_ACLTYPE_POSIX)
 		return (-EOPNOTSUPP);
 
@@ -1394,7 +1337,7 @@ __zpl_xattr_acl_set_default(zidmap_t *mnt_ns,
 		if (IS_ERR(acl))
 			return (PTR_ERR(acl));
 		else if (acl) {
-			error = zpl_posix_acl_valid(ip, acl);
+			error = posix_acl_valid(ip->i_sb->s_user_ns, acl);
 			if (error) {
 				zpl_posix_acl_release(acl);
 				return (error);
@@ -1418,41 +1361,25 @@ ZPL_XATTR_SET_WRAPPER(zpl_xattr_acl_set_default);
  * whole name and reject anything that has .name only as prefix.
  */
 static xattr_handler_t zpl_xattr_acl_access_handler = {
-#ifdef HAVE_XATTR_HANDLER_NAME
 	.name	= XATTR_NAME_POSIX_ACL_ACCESS,
-#else
-	.prefix	= XATTR_NAME_POSIX_ACL_ACCESS,
-#endif
 	.list	= zpl_xattr_acl_list_access,
 	.get	= zpl_xattr_acl_get_access,
 	.set	= zpl_xattr_acl_set_access,
-#if defined(HAVE_XATTR_LIST_SIMPLE) || \
-    defined(HAVE_XATTR_LIST_DENTRY) || \
-    defined(HAVE_XATTR_LIST_HANDLER)
 	.flags	= ACL_TYPE_ACCESS,
-#endif
 };
 
 /*
  * ACL default xattr namespace handlers.
  *
- * Use .name instead of .prefix when available. xattr_resolve_name will match
- * whole name and reject anything that has .name only as prefix.
+ * Use .name instead of .prefix. xattr_resolve_name will match whole name and
+ * reject anything that has .name only as prefix.
  */
 static xattr_handler_t zpl_xattr_acl_default_handler = {
-#ifdef HAVE_XATTR_HANDLER_NAME
 	.name	= XATTR_NAME_POSIX_ACL_DEFAULT,
-#else
-	.prefix	= XATTR_NAME_POSIX_ACL_DEFAULT,
-#endif
 	.list	= zpl_xattr_acl_list_default,
 	.get	= zpl_xattr_acl_get_default,
 	.set	= zpl_xattr_acl_set_default,
-#if defined(HAVE_XATTR_LIST_SIMPLE) || \
-    defined(HAVE_XATTR_LIST_DENTRY) || \
-    defined(HAVE_XATTR_LIST_HANDLER)
 	.flags	= ACL_TYPE_DEFAULT,
-#endif
 };
 
 #endif /* CONFIG_FS_POSIX_ACL */
@@ -1517,24 +1444,15 @@ zpl_xattr_permission(xattr_filldir_t *xf, const char *name, int name_len)
 	}
 
 	if (handler->list) {
-#if defined(HAVE_XATTR_LIST_SIMPLE)
 		if (!handler->list(d))
 			return (XAPERM_DENY);
-#elif defined(HAVE_XATTR_LIST_DENTRY)
-		if (!handler->list(d, NULL, 0, name, name_len, 0))
-			return (XAPERM_DENY);
-#elif defined(HAVE_XATTR_LIST_HANDLER)
-		if (!handler->list(handler, d, NULL, 0, name, name_len))
-			return (XAPERM_DENY);
-#endif
 	}
 
 	return (perm);
 }
 
-#if defined(CONFIG_FS_POSIX_ACL) && \
-	(!defined(HAVE_POSIX_ACL_RELEASE) || \
-		defined(HAVE_POSIX_ACL_RELEASE_GPL_ONLY))
+#ifdef CONFIG_FS_POSIX_ACL
+
 struct acl_rel_struct {
 	struct acl_rel_struct *next;
 	struct posix_acl *acl;
