@@ -133,7 +133,7 @@ static void
 dsl_deadlist_load_tree(dsl_deadlist_t *dl)
 {
 	zap_cursor_t zc;
-	zap_attribute_t za;
+	zap_attribute_t *za;
 	int error;
 
 	ASSERT(MUTEX_HELD(&dl->dl_lock));
@@ -159,20 +159,21 @@ dsl_deadlist_load_tree(dsl_deadlist_t *dl)
 	if (dl->dl_havetree)
 		return;
 
+	za = zap_attribute_alloc();
 	avl_create(&dl->dl_tree, dsl_deadlist_compare,
 	    sizeof (dsl_deadlist_entry_t),
 	    offsetof(dsl_deadlist_entry_t, dle_node));
 	for (zap_cursor_init(&zc, dl->dl_os, dl->dl_object);
-	    (error = zap_cursor_retrieve(&zc, &za)) == 0;
+	    (error = zap_cursor_retrieve(&zc, za)) == 0;
 	    zap_cursor_advance(&zc)) {
 		dsl_deadlist_entry_t *dle = kmem_alloc(sizeof (*dle), KM_SLEEP);
-		dle->dle_mintxg = zfs_strtonum(za.za_name, NULL);
+		dle->dle_mintxg = zfs_strtonum(za->za_name, NULL);
 
 		/*
 		 * Prefetch all the bpobj's so that we do that i/o
 		 * in parallel.  Then open them all in a second pass.
 		 */
-		dle->dle_bpobj.bpo_object = za.za_first_integer;
+		dle->dle_bpobj.bpo_object = za->za_first_integer;
 		dmu_prefetch_dnode(dl->dl_os, dle->dle_bpobj.bpo_object,
 		    ZIO_PRIORITY_SYNC_READ);
 
@@ -180,6 +181,7 @@ dsl_deadlist_load_tree(dsl_deadlist_t *dl)
 	}
 	VERIFY3U(error, ==, ENOENT);
 	zap_cursor_fini(&zc);
+	zap_attribute_free(za);
 
 	for (dsl_deadlist_entry_t *dle = avl_first(&dl->dl_tree);
 	    dle != NULL; dle = AVL_NEXT(&dl->dl_tree, dle)) {
@@ -207,7 +209,7 @@ static void
 dsl_deadlist_load_cache(dsl_deadlist_t *dl)
 {
 	zap_cursor_t zc;
-	zap_attribute_t za;
+	zap_attribute_t *za;
 	int error;
 
 	ASSERT(MUTEX_HELD(&dl->dl_lock));
@@ -221,26 +223,28 @@ dsl_deadlist_load_cache(dsl_deadlist_t *dl)
 	avl_create(&dl->dl_cache, dsl_deadlist_cache_compare,
 	    sizeof (dsl_deadlist_cache_entry_t),
 	    offsetof(dsl_deadlist_cache_entry_t, dlce_node));
+	za = zap_attribute_alloc();
 	for (zap_cursor_init(&zc, dl->dl_os, dl->dl_object);
-	    (error = zap_cursor_retrieve(&zc, &za)) == 0;
+	    (error = zap_cursor_retrieve(&zc, za)) == 0;
 	    zap_cursor_advance(&zc)) {
-		if (za.za_first_integer == empty_bpobj)
+		if (za->za_first_integer == empty_bpobj)
 			continue;
 		dsl_deadlist_cache_entry_t *dlce =
 		    kmem_zalloc(sizeof (*dlce), KM_SLEEP);
-		dlce->dlce_mintxg = zfs_strtonum(za.za_name, NULL);
+		dlce->dlce_mintxg = zfs_strtonum(za->za_name, NULL);
 
 		/*
 		 * Prefetch all the bpobj's so that we do that i/o
 		 * in parallel.  Then open them all in a second pass.
 		 */
-		dlce->dlce_bpobj = za.za_first_integer;
+		dlce->dlce_bpobj = za->za_first_integer;
 		dmu_prefetch_dnode(dl->dl_os, dlce->dlce_bpobj,
 		    ZIO_PRIORITY_SYNC_READ);
 		avl_add(&dl->dl_cache, dlce);
 	}
 	VERIFY3U(error, ==, ENOENT);
 	zap_cursor_fini(&zc);
+	zap_attribute_free(za);
 
 	for (dsl_deadlist_cache_entry_t *dlce = avl_first(&dl->dl_cache);
 	    dlce != NULL; dlce = AVL_NEXT(&dl->dl_cache, dlce)) {
@@ -295,30 +299,33 @@ dsl_deadlist_iterate(dsl_deadlist_t *dl, deadlist_iter_t func, void *args)
 	}
 }
 
-void
+int
 dsl_deadlist_open(dsl_deadlist_t *dl, objset_t *os, uint64_t object)
 {
 	dmu_object_info_t doi;
+	int err;
 
 	ASSERT(!dsl_deadlist_is_open(dl));
 
 	mutex_init(&dl->dl_lock, NULL, MUTEX_DEFAULT, NULL);
 	dl->dl_os = os;
 	dl->dl_object = object;
-	VERIFY0(dmu_bonus_hold(os, object, dl, &dl->dl_dbuf));
+	err = dmu_bonus_hold(os, object, dl, &dl->dl_dbuf);
+	if (err != 0)
+		return (err);
 	dmu_object_info_from_db(dl->dl_dbuf, &doi);
 	if (doi.doi_type == DMU_OT_BPOBJ) {
 		dmu_buf_rele(dl->dl_dbuf, dl);
 		dl->dl_dbuf = NULL;
 		dl->dl_oldfmt = B_TRUE;
-		VERIFY0(bpobj_open(&dl->dl_bpobj, os, object));
-		return;
+		return (bpobj_open(&dl->dl_bpobj, os, object));
 	}
 
 	dl->dl_oldfmt = B_FALSE;
 	dl->dl_phys = dl->dl_dbuf->db_data;
 	dl->dl_havetree = B_FALSE;
 	dl->dl_havecache = B_FALSE;
+	return (0);
 }
 
 boolean_t
@@ -381,7 +388,7 @@ dsl_deadlist_free(objset_t *os, uint64_t dlobj, dmu_tx_t *tx)
 {
 	dmu_object_info_t doi;
 	zap_cursor_t zc;
-	zap_attribute_t za;
+	zap_attribute_t *za;
 	int error;
 
 	VERIFY0(dmu_object_info(os, dlobj, &doi));
@@ -390,10 +397,11 @@ dsl_deadlist_free(objset_t *os, uint64_t dlobj, dmu_tx_t *tx)
 		return;
 	}
 
+	za = zap_attribute_alloc();
 	for (zap_cursor_init(&zc, os, dlobj);
-	    (error = zap_cursor_retrieve(&zc, &za)) == 0;
+	    (error = zap_cursor_retrieve(&zc, za)) == 0;
 	    zap_cursor_advance(&zc)) {
-		uint64_t obj = za.za_first_integer;
+		uint64_t obj = za->za_first_integer;
 		if (obj == dmu_objset_pool(os)->dp_empty_bpobj)
 			bpobj_decr_empty(os, tx);
 		else
@@ -401,6 +409,7 @@ dsl_deadlist_free(objset_t *os, uint64_t dlobj, dmu_tx_t *tx)
 	}
 	VERIFY3U(error, ==, ENOENT);
 	zap_cursor_fini(&zc);
+	zap_attribute_free(za);
 	VERIFY0(dmu_object_free(os, dlobj, tx));
 }
 
@@ -474,7 +483,7 @@ dsl_deadlist_insert(dsl_deadlist_t *dl, const blkptr_t *bp, boolean_t bp_freed,
 	dl->dl_phys->dl_comp += sign * BP_GET_PSIZE(bp);
 	dl->dl_phys->dl_uncomp += sign * BP_GET_UCSIZE(bp);
 
-	dle_tofind.dle_mintxg = bp->blk_birth;
+	dle_tofind.dle_mintxg = BP_GET_LOGICAL_BIRTH(bp);
 	dle = avl_find(&dl->dl_tree, &dle_tofind, &where);
 	if (dle == NULL)
 		dle = avl_nearest(&dl->dl_tree, where, AVL_BEFORE);
@@ -483,7 +492,7 @@ dsl_deadlist_insert(dsl_deadlist_t *dl, const blkptr_t *bp, boolean_t bp_freed,
 
 	if (dle == NULL) {
 		zfs_panic_recover("blkptr at %p has invalid BLK_BIRTH %llu",
-		    bp, (longlong_t)bp->blk_birth);
+		    bp, (longlong_t)BP_GET_LOGICAL_BIRTH(bp));
 		dle = avl_first(&dl->dl_tree);
 	}
 
@@ -680,7 +689,7 @@ dsl_deadlist_regenerate(objset_t *os, uint64_t dlobj,
 	dsl_deadlist_t dl = { 0 };
 	dsl_pool_t *dp = dmu_objset_pool(os);
 
-	dsl_deadlist_open(&dl, os, dlobj);
+	VERIFY0(dsl_deadlist_open(&dl, os, dlobj));
 	if (dl.dl_oldfmt) {
 		dsl_deadlist_close(&dl);
 		return;
@@ -875,8 +884,8 @@ dsl_deadlist_merge(dsl_deadlist_t *dl, uint64_t obj, dmu_tx_t *tx)
 		return;
 	}
 
-	za = kmem_alloc(sizeof (*za), KM_SLEEP);
-	pza = kmem_alloc(sizeof (*pza), KM_SLEEP);
+	za = zap_attribute_alloc();
+	pza = zap_attribute_alloc();
 
 	mutex_enter(&dl->dl_lock);
 	/*
@@ -913,8 +922,8 @@ dsl_deadlist_merge(dsl_deadlist_t *dl, uint64_t obj, dmu_tx_t *tx)
 	dmu_buf_rele(bonus, FTAG);
 	mutex_exit(&dl->dl_lock);
 
-	kmem_free(za, sizeof (*za));
-	kmem_free(pza, sizeof (*pza));
+	zap_attribute_free(za);
+	zap_attribute_free(pza);
 }
 
 /*
@@ -1039,8 +1048,7 @@ dsl_livelist_iterate(void *arg, const blkptr_t *bp, boolean_t bp_freed,
 		ASSERT3U(BP_GET_PSIZE(bp), ==, BP_GET_PSIZE(&found->le_bp));
 		ASSERT3U(BP_GET_CHECKSUM(bp), ==,
 		    BP_GET_CHECKSUM(&found->le_bp));
-		ASSERT3U(BP_PHYSICAL_BIRTH(bp), ==,
-		    BP_PHYSICAL_BIRTH(&found->le_bp));
+		ASSERT3U(BP_GET_BIRTH(bp), ==, BP_GET_BIRTH(&found->le_bp));
 	}
 	if (bp_freed) {
 		if (found == NULL) {
