@@ -6,7 +6,7 @@
  * You may not use this file except in compliance with the License.
  *
  * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
- * or http://www.opensolaris.org/os/licensing.
+ * or https://opensource.org/licenses/CDDL-1.0.
  * See the License for the specific language governing permissions
  * and limitations under the License.
  *
@@ -59,7 +59,7 @@ dodefault(zfs_prop_t prop, int intsz, int numints, void *buf)
 	if (zfs_prop_get_type(prop) == PROP_TYPE_STRING) {
 		if (intsz != 1)
 			return (SET_ERROR(EOVERFLOW));
-		(void) strncpy(buf, zfs_prop_default_string(prop),
+		(void) strlcpy(buf, zfs_prop_default_string(prop),
 		    numints);
 	} else {
 		if (intsz != 8 || numints < 1)
@@ -102,7 +102,7 @@ dsl_prop_get_dd(dsl_dir_t *dd, const char *propname,
 		setpoint[0] = '\0';
 
 	prop = zfs_name_to_prop(propname);
-	inheritable = (prop == ZPROP_INVAL || zfs_prop_inheritable(prop));
+	inheritable = (prop == ZPROP_USERPROP || zfs_prop_inheritable(prop));
 	inheritstr = kmem_asprintf("%s%s", propname, ZPROP_INHERIT_SUFFIX);
 	recvdstr = kmem_asprintf("%s%s", propname, ZPROP_RECVD_SUFFIX);
 	iuvstr = kmem_asprintf("%s%s", propname, ZPROP_IUV_SUFFIX);
@@ -123,7 +123,7 @@ dsl_prop_get_dd(dsl_dir_t *dd, const char *propname,
 		/* Check for a iuv value. */
 		err = zap_lookup(mos, dsl_dir_phys(dd)->dd_props_zapobj,
 		    iuvstr, intsz, numints, buf);
-		if (dsl_prop_known_index(zfs_name_to_prop(propname),
+		if (err == 0 && dsl_prop_known_index(prop,
 		    *(uint64_t *)buf) != 1)
 			err = ENOENT;
 		if (err != ENOENT) {
@@ -196,7 +196,7 @@ dsl_prop_get_ds(dsl_dataset_t *ds, const char *propname,
 	uint64_t zapobj;
 
 	ASSERT(dsl_pool_config_held(ds->ds_dir->dd_pool));
-	inheritable = (prop == ZPROP_INVAL || zfs_prop_inheritable(prop));
+	inheritable = (prop == ZPROP_USERPROP || zfs_prop_inheritable(prop));
 	zapobj = dsl_dataset_phys(ds)->ds_props_obj;
 
 	if (zapobj != 0) {
@@ -662,7 +662,7 @@ dsl_prop_changed_notify(dsl_pool_t *dp, uint64_t ddobj,
 	}
 	mutex_exit(&dd->dd_lock);
 
-	za = kmem_alloc(sizeof (zap_attribute_t), KM_SLEEP);
+	za = zap_attribute_alloc();
 	for (zap_cursor_init(&zc, mos,
 	    dsl_dir_phys(dd)->dd_child_dir_zapobj);
 	    zap_cursor_retrieve(&zc, za) == 0;
@@ -670,7 +670,7 @@ dsl_prop_changed_notify(dsl_pool_t *dp, uint64_t ddobj,
 		dsl_prop_changed_notify(dp, za->za_first_integer,
 		    propname, value, FALSE);
 	}
-	kmem_free(za, sizeof (zap_attribute_t));
+	zap_attribute_free(za);
 	zap_cursor_fini(&zc);
 	dsl_dir_rele(dd, FTAG);
 }
@@ -698,6 +698,10 @@ dsl_prop_set_iuv(objset_t *mos, uint64_t zapobj, const char *propname,
 		    *(uint64_t *)value == ZFS_REDUNDANT_METADATA_NONE)
 			iuv = B_TRUE;
 		break;
+	case ZFS_PROP_SNAPDIR:
+		if (*(uint64_t *)value == ZFS_SNAPDIR_DISABLED)
+			iuv = B_TRUE;
+		break;
 	default:
 		break;
 	}
@@ -823,7 +827,7 @@ dsl_prop_set_sync_impl(dsl_dataset_t *ds, const char *propname,
 		ASSERT(err == 0 || err == ENOENT);
 		err = zap_remove(mos, zapobj, inheritstr, tx);
 		ASSERT(err == 0 || err == ENOENT);
-		fallthrough;
+		zfs_fallthrough;
 	case (ZPROP_SRC_NONE | ZPROP_SRC_RECEIVED):
 		/*
 		 * remove propname$recvd
@@ -956,7 +960,7 @@ dsl_props_set_check(void *arg, dmu_tx_t *tx)
 			return (SET_ERROR(ENAMETOOLONG));
 		}
 		if (nvpair_type(elem) == DATA_TYPE_STRING) {
-			char *valstr = fnvpair_value_string(elem);
+			const char *valstr = fnvpair_value_string(elem);
 			if (strlen(valstr) >= (version <
 			    SPA_VERSION_STMF_PROP ?
 			    ZAP_OLDMAXVALUELEN : ZAP_MAXVALUELEN)) {
@@ -1061,11 +1065,11 @@ dsl_prop_get_all_impl(objset_t *mos, uint64_t propobj,
     const char *setpoint, dsl_prop_getflags_t flags, nvlist_t *nv)
 {
 	zap_cursor_t zc;
-	zap_attribute_t za;
+	zap_attribute_t *za = zap_attribute_alloc();
 	int err = 0;
 
 	for (zap_cursor_init(&zc, mos, propobj);
-	    (err = zap_cursor_retrieve(&zc, &za)) == 0;
+	    (err = zap_cursor_retrieve(&zc, za)) == 0;
 	    zap_cursor_advance(&zc)) {
 		nvlist_t *propval;
 		zfs_prop_t prop;
@@ -1075,7 +1079,7 @@ dsl_prop_get_all_impl(objset_t *mos, uint64_t propobj,
 		const char *propname;
 		const char *source;
 
-		suffix = strchr(za.za_name, '$');
+		suffix = strchr(za->za_name, '$');
 
 		if (suffix == NULL) {
 			/*
@@ -1085,7 +1089,7 @@ dsl_prop_get_all_impl(objset_t *mos, uint64_t propobj,
 			if (flags & DSL_PROP_GET_RECEIVED)
 				continue;
 
-			propname = za.za_name;
+			propname = za->za_name;
 			source = setpoint;
 
 			/* Skip if iuv entries are preset. */
@@ -1102,8 +1106,8 @@ dsl_prop_get_all_impl(objset_t *mos, uint64_t propobj,
 			if (flags & DSL_PROP_GET_LOCAL)
 				continue;
 
-			(void) strncpy(buf, za.za_name, (suffix - za.za_name));
-			buf[suffix - za.za_name] = '\0';
+			(void) strlcpy(buf, za->za_name,
+			    MIN(sizeof (buf), suffix - za->za_name + 1));
 			propname = buf;
 
 			if (!(flags & DSL_PROP_GET_RECEIVED)) {
@@ -1128,14 +1132,14 @@ dsl_prop_get_all_impl(objset_t *mos, uint64_t propobj,
 			source = ((flags & DSL_PROP_GET_INHERITING) ?
 			    setpoint : ZPROP_SOURCE_VAL_RECVD);
 		} else if (strcmp(suffix, ZPROP_IUV_SUFFIX) == 0) {
-			(void) strlcpy(buf, za.za_name,
-			    MIN(sizeof (buf), suffix - za.za_name + 1));
+			(void) strlcpy(buf, za->za_name,
+			    MIN(sizeof (buf), suffix - za->za_name + 1));
 			propname = buf;
 			source = setpoint;
 			prop = zfs_name_to_prop(propname);
 
 			if (dsl_prop_known_index(prop,
-			    za.za_first_integer) != 1)
+			    za->za_first_integer) != 1)
 				continue;
 		} else {
 			/*
@@ -1148,12 +1152,12 @@ dsl_prop_get_all_impl(objset_t *mos, uint64_t propobj,
 		prop = zfs_name_to_prop(propname);
 
 		/* Skip non-inheritable properties. */
-		if ((flags & DSL_PROP_GET_INHERITING) && prop != ZPROP_INVAL &&
-		    !zfs_prop_inheritable(prop))
+		if ((flags & DSL_PROP_GET_INHERITING) &&
+		    prop != ZPROP_USERPROP && !zfs_prop_inheritable(prop))
 			continue;
 
 		/* Skip properties not valid for this type. */
-		if ((flags & DSL_PROP_GET_SNAPSHOT) && prop != ZPROP_INVAL &&
+		if ((flags & DSL_PROP_GET_SNAPSHOT) && prop != ZPROP_USERPROP &&
 		    !zfs_prop_valid_for_type(prop, ZFS_TYPE_SNAPSHOT, B_FALSE))
 			continue;
 
@@ -1162,28 +1166,28 @@ dsl_prop_get_all_impl(objset_t *mos, uint64_t propobj,
 			continue;
 
 		VERIFY(nvlist_alloc(&propval, NV_UNIQUE_NAME, KM_SLEEP) == 0);
-		if (za.za_integer_length == 1) {
+		if (za->za_integer_length == 1) {
 			/*
 			 * String property
 			 */
-			char *tmp = kmem_alloc(za.za_num_integers,
+			char *tmp = kmem_alloc(za->za_num_integers,
 			    KM_SLEEP);
 			err = zap_lookup(mos, propobj,
-			    za.za_name, 1, za.za_num_integers, tmp);
+			    za->za_name, 1, za->za_num_integers, tmp);
 			if (err != 0) {
-				kmem_free(tmp, za.za_num_integers);
+				kmem_free(tmp, za->za_num_integers);
 				break;
 			}
 			VERIFY(nvlist_add_string(propval, ZPROP_VALUE,
 			    tmp) == 0);
-			kmem_free(tmp, za.za_num_integers);
+			kmem_free(tmp, za->za_num_integers);
 		} else {
 			/*
 			 * Integer property
 			 */
-			ASSERT(za.za_integer_length == 8);
+			ASSERT(za->za_integer_length == 8);
 			(void) nvlist_add_uint64(propval, ZPROP_VALUE,
-			    za.za_first_integer);
+			    za->za_first_integer);
 		}
 
 		VERIFY(nvlist_add_string(propval, ZPROP_SOURCE, source) == 0);
@@ -1191,6 +1195,7 @@ dsl_prop_get_all_impl(objset_t *mos, uint64_t propobj,
 		nvlist_free(propval);
 	}
 	zap_cursor_fini(&zc);
+	zap_attribute_free(za);
 	if (err == ENOENT)
 		err = 0;
 	return (err);
